﻿---------------------------------------------------------------------------------
--
-- Prat - A framework for World of Warcraft chat mods
--
-- Copyright (C) 2006-2007  Prat Development Team
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc., 
-- 51 Franklin Street, Fifth Floor, 
-- Boston, MA  02110-1301, USA.
--
--
-------------------------------------------------------------------------------



--[[
Name: PratChatTabs
Revision: $Revision: 44429 $
Author(s): Sylvanaar (sylvanaar@mindspring.com)
Website: http://www.wowace.com/files/index.php?path=Prat/
Documentation: http://www.wowace.com/wiki/Prat/Integrated_Modules#ChatTabs
Subversion: http://svn.wowace.com/wowace/trunk/Prat/
Discussions: http://groups.google.com/group/wow-prat
Issues and feature requests: http://code.google.com/p/prat/issues/list
Description: Module for Prat that adds chat window tab options.
Dependencies: Prat
]]

-- Get Utility Libraries
local util, DBG, CLR = GetPratUtils()

-- set prat module name
local PRAT_MODULE = Prat:RequestModuleName("PratChatTabs")

if PRAT_MODULE == nil then 
    return 
end

-- define localized strings
local L = AceLibrary("AceLocale-2.2"):new(PRAT_MODULE)

L:RegisterTranslations("enUS", function() return {
    ["ChatTabs"] = true,
    ["Chat window tab options."] = true,
    ["SetMode"] = true,
    ["Toggle mode"] = true,
    ["SetAll"] = true,
    ["Set chat tabs for all windows."] = true,
    ["Individual"] = true,
    ["Toggle chat tabs for each window."] = true,
    ["Chat%dTab"] = true,
    ["Toggles chat tab options for chat window %d."] = true,
    ["Active Alpha"] = true,
    ["Sets alpha of chat tab for active chat frame."] = true,
    ["Not Active Alpha"] = true,
    ["Sets alpha of chat tab for not active chat frame."] = true,
    ["All"] = true,
    ["Individual"] = true,
    ["Always Visible"] = true,
    ["Hidden"] = true,
    ["Default"] = true,
    ["flashselected_name"] = "Flash Selected Tab",
    ["flashselected_desc"] = "Toggle flashing the active chat tab on whispers.",
    ["disableflash_name"] =  "Disable Flashing",
    ["disableflash_desc"] =  "Disable flashing of the chat tabs."
} end)

L:RegisterTranslations("deDE", function() return {
    ["ChatTabs"] = "Chat Tasten",
    ["Chat window tab options."] = "Chat Tasten Optionen",
    ["SetMode"] = "Setze Modus",
    ["Toggle mode"] = "Schalte Modus",
    ["SetAll"] = "Setze Alle",
    ["Set chat tabs for all windows."] = "Setze Chat Tastens f\195\188r alle Fenster",
    ["Individual"] = "Individuell",
    ["Toggle chat tabs for each window."] = "Aktiviere Chat Tasten f\195\188r jedes Fenster",
    ["Chat%dTab"] = "Chat %d Taste",
    ["Toggles chat tab options for chat window %d."] = "Aktiviert Chat Tasten Optionen f\195\188r Chat Fenster %d.",
    ["Active Alpha"] = "Aktive Helligkeit",
    ["Sets alpha of chat tab for active chat frame."] = "Stellt die Helligkeit f\195\188r aktive Chattasten Fenster ein.",
    ["Not Active Alpha"] = "Keine Aktive Helligkeit",
    ["Sets alpha of chat tab for not active chat frame."] = "Stellt die Helligkeit f\195\188r nicht aktive Chattasten ein.",
    ["All"] = "Alle",
    ["Individual"] = "Individuell",
    ["Always Visible"] = "Immer sichtbar",
    ["Hidden"] = "Versteckt",
    ["Default"] = "Grundeinstellung",
    ["flashselected_name"] = "Aufblitzen der benutzten Taste",
    ["flashselected_desc"] = "Aktiviert Aufblitzen der Chat Taste vom gegew\195\164rtig benutzen Chatfenster wenn eine gel\195\188sterte Nachricht ankommt.",
} end)

L:RegisterTranslations("esES", function() return {
    ["ChatTabs"] = "Pesta\195\177as de Chat",
    ["Chat window tab options."] = "Opciones de las pesta\195\177as de chat",
    ["SetMode"] = "Establecer Modo",
    ["Toggle mode"] = "Cambia el modo",
    ["SetAll"] = "Establecer Todos",
    ["Set chat tabs for all windows."] = "Establece las pesta\195\177as de chat para todas las ventanas",
    ["Individual"] = "Individual",
    ["Toggle chat tabs for each window."] = "Cambia las pesta\195\177as de chat para cada ventana",
    ["Chat%dTab"] = "Pesta\195\177a de Chat %d",
    ["Toggles chat tab options for chat window %d."] = "Determina las opciones de la pesta\195\177a de chat de la ventana de chat %d",
    ["Active Alpha"] = "Transparencia Activos",
    ["Sets alpha of chat tab for active chat frame."] = "Establece el \195\173ndice de transparencia de la pesta\195\177a de chat para los marcos de chat activos.",
    ["Not Active Alpha"] = "Transparencia No Activos",
    ["Sets alpha of chat tab for not active chat frame."] = "Establece el \195\173ndice de transparencia de la pesta\195\177a de chat para los marcos de chat no activos.",
    ["All"] = "Todos",
    ["Individual"] = "Individual",
    ["Always Visible"] = "Siempre Visible",
    ["Hidden"] = "Oculto",
    ["Default"] = "Por Defecto",
    ["flashselected_name"] = "Destello en la Pesta\195\177a Seleccionada",
    ["flashselected_desc"] = "Determina si se muestra un destello en la pesta\195\177a del chat activo cuando te susurran",
} end)

L:RegisterTranslations("koKR", function() return {
    ["ChatTabs"] = "대화탭",
    ["Chat window tab options."] = "대화탭 설정",
    ["SetMode"] = "모드설정",
    ["Toggle mode"] = "모드 전환",
    ["SetAll"] = "모두설정",
    ["Set chat tabs for all windows."] = "모든 창에대한 대화탭을 설정합니다.",
    ["Individual"] = "개별",
    ["Toggle chat tabs for each window."] = "각 창에 대한 대화 탭을 전환합니다.",
    ["Chat%dTab"] = "대화%d탭",
    ["Toggles chat tab options for chat window %d."] = "%d 대화창에 대한 대화탭 설정을 전환합니다.",
    ["Active Alpha"] = "활성화 투명도",
    ["Sets alpha of chat tab for active chat frame."] = "활성화된 대화창에 대한 대화탭의 투명도를 설정합니다.",
    ["Not Active Alpha"] = "비활성화 투명도",
    ["Sets alpha of chat tab for not active chat frame."] = "비활성화 대화창에 대한 대화탭의 투명도를 설정합니다.",
    ["All"] = "모두",
    ["Individual"] = "개별",
    ["Always Visible"] = "항상 표시",
    ["Hidden"] = "숨김",
    ["Default"] = "기본값",
    ["flashselected_name"] = "선택된 탭 강조",
    ["flashselected_desc"] = "귓속말에 활성화된 대화탭 강조를 전환합니다.",
    ["disableflash_name"] =  "번쩍임 미사용",
    ["disableflash_desc"] =  "대화탭의 번쩍임을 사용하지 않습니다."
} end)

L:RegisterTranslations("frFR", function() return {
    ["ChatTabs"] = "Onglets",
    ["Chat window tab options."] = "Options des onglets.",
    ["SetMode"] = "Mode de r\195\169glage",
    ["Toggle mode"] = "Change le mode de r\195\169glage.",
    ["SetAll"] = "Toutes les fen\195\170tres",
    ["Set chat tabs for all windows."] = "R\195\168gle les onglets pour toutes les fen\195\170tres à la fois.",
    ["Individual"] = "Chaque fenêtre",
    ["Toggle chat tabs for each window."] = "R\195\168gle les onglets fen\195\170tre par fen\195\170re.",
    ["Chat%dTab"] = "Onglets de la fen\195\170tre %d",
    ["Toggles chat tab options for chat window %d."] = "Change les options d'onglet de la fen\195\170tre %d.",
    ["Active Alpha"] = "Transparence lorsque actif",
    ["Sets alpha of chat tab for active chat frame."] = "R\195\168gle la transparence des onglets de la fen\195\170tre active.",
    ["Not Active Alpha"] = "Transparence lorsque inactif",
    ["Sets alpha of chat tab for not active chat frame."] = "R\195\168gle la transparence des onglets des fen\195\170tres inactives.",
    ["All"] = "Toutes les fen\195\170tres",
    ["Individual"] = "Fen\195\170tre par fen\195\170tre",
    ["Always Visible"] = "Toujours visible",
    ["Hidden"] = "Cach\195\169s",
    ["Default"] = "Par d\195\169faut",
    ["flashselected_name"] = "Faire clignoter l'onglet actif",
    ["flashselected_desc"] = "Active ou désactive le clignotement de l'onglet actif lorsqu'un message est recu.",
} end)

--[[
	Chinese Local : CWDG Translation Team 昏睡墨鱼 (Thomas Mo)
	CWDG site: http://Cwowaddon.com
	$Rev: 44429 $
	$Date: 2007-07-21 09:41:41 -0400 (Sat, 21 Jul 2007) $
]]

L:RegisterTranslations("zhCN", function() return {
    ["ChatTabs"] = "聊天标签",
    ["Chat window tab options."] = "聊天窗口标签选项",
    ["SetMode"] = "设置模式",
    ["Toggle mode"] = "切换模式",
    ["SetAll"] = "设置全部",
    ["Set chat tabs for all windows."] = "设置所有聊天窗口的标签",
    ["Individual"] = "独立",
    ["Toggle chat tabs for each window."] = "切换个别聊天窗口的标签",
    ["Chat%dTab"] = "聊天窗口 %d 标签",
    ["Toggles chat tab options for chat window %d."] = "切换聊天窗口 %d 标签选项",
    ["Active Alpha"] = "激活透明度",
    ["Sets alpha of chat tab for active chat frame."] = "设置激活的聊天窗口标签透明度",
    ["Not Active Alpha"] = "未激活透明度",
    ["Sets alpha of chat tab for not active chat frame."] = "设置未激活的聊天窗口标签透明度",
    ["All"] = "全部",
    ["Individual"] = "独立",
    ["Always Visible"] = "永远可见",
    ["Hidden"] = "隐藏",
    ["Default"] = "默认",
    ["flashselected_name"] = "选择的标签闪光",
    ["flashselected_desc"] = "有密语的聊天标签闪光",
    ["disableflash_name"] =  "取消闪光",
    ["disableflash_desc"] =  "取消聊天标签闪光"
} end)

L:RegisterTranslations("zhTW", function() return {
    ["ChatTabs"] = "聊天標籤",
    ["Chat window tab options."] = "聊天視窗標籤選項",
    ["SetMode"] = "設定模式",
    ["Toggle mode"] = "啟用模式",
    ["SetAll"] = "設定全部",
    ["Set chat tabs for all windows."] = "設定所有聊天視窗的標籤",
    ["Individual"] = "個別",
    ["Toggle chat tabs for each window."] = "設定個別聊天視窗的標籤",
    ["Chat%dTab"] = "聊天 %d 標籤",
    ["Toggles chat tab options for chat window %d."] = "啟用聊天視窗 %d 標籤選項",
    ["Active Alpha"] = "啟用的透明度",
    ["Sets alpha of chat tab for active chat frame."] = "設定啟用的聊天視窗標籤透明度",
    ["Not Active Alpha"] = "未啟用的透明度",
    ["Sets alpha of chat tab for not active chat frame."] = "設定未啟用的聊天視窗標籤透明度",
    ["All"] = "全部",
    ["Individual"] = "個別",
    ["Always Visible"] = "永遠可見",
    ["Hidden"] = "隱藏",
    ["Default"] = "預設",
    ["flashselected_name"] = "被選擇標籤閃光",
    ["flashselected_desc"] = "啟用有密語的聊天標籤閃光",
    ["disableflash_name"] =  "取消閃光",
    ["disableflash_desc"] =  "取消聊天標籤閃光"
} end)

-- get prat module categories
local cat = Prat.Categories

-- create prat module
Prat_ChatTabs = Prat:NewModule(PRAT_MODULE)
Prat_ChatTabs.revision = tonumber(string.sub("$Revision: 44429 $", 12, -3))

-- define key module values
Prat_ChatTabs.moduleName = L["ChatTabs"]
Prat_ChatTabs.moduleDesc = L["Chat window tab options."]
Prat_ChatTabs.consoleName = "chattabs"
Prat_ChatTabs.guiName = L["ChatTabs"]
Prat_ChatTabs.Categories = { cat.FRAME }

-- define the default db values
Prat_ChatTabs.defaultDB = {
        on = false,
        notactivealpha = 0.5,
        activealpha = 1,
        mode = "ALL",
        cf = {"ALWAYS", "ALWAYS", "ALWAYS", "ALWAYS", "ALWAYS", "ALWAYS", "ALWAYS"},
        cfall = "DEFAULT",
        flashselected = false,
        disableflash = false,
}

-- create a moduleOptions stub (for setting self.moduleOptions)
Prat_ChatTabs.moduleOptions = {}
Prat_ChatTabs.toggleOptions = { disableflash=211 }

-- add module options not covered by templates
function Prat_ChatTabs:GetModuleOptions()
     self.moduleOptions = {
        name = L["ChatTabs"],
        desc = L["Chat window tab options."],
        type = "group",
        args = {
            mode = {
                name = L["SetMode"],
                desc = L["Toggle mode"],
                type = "text",
                order = 100,
                get = function() return self.db.profile.mode end,
                set = function(v) self.db.profile.mode = v self:UpdateAllTabs() end,
                validate = {["ALL"] = L["All"], ["INDIVIDUAL"] = L["Individual"]},
            },
            all = {
                name = L["SetAll"],
                desc = L["Set chat tabs for all windows."],
                type = "text",
                disabled = function() if self.db.profile.mode == "INDIVIDUAL" then return true else return false end end,
                order = 110,
                get = function() return self.db.profile.cfall end,
                set = function(v) self.db.profile.cfall = v self:UpdateAllTabs() end,
                validate = {["ALWAYS"] = L["Always Visible"], ["HIDDEN"] = L["Hidden"], ["DEFAULT"] = L["Default"]},
            },
            individual = {
                name = L["Individual"],
                desc = L["Toggle chat tabs for each window."],
                type = "group",
                disabled = function() if self.db.profile.mode == "ALL" then return true else return false end end,
                order = 120,
                args = {
                    chat1 = {
                        name = string.format(L["Chat%dTab"], 1),
                        desc = string.format(L["Toggles chat tab options for chat window %d."], 1),
                        type = "text",
                        get = function() return self.db.profile.cf[1] end,
                        set = function(v)
                            self.db.profile.cf[1] = v
                            self:UpdateAllTabs()
                        end,
                        validate = {["ALWAYS"] = L["Always Visible"], ["HIDDEN"] = L["Hidden"], ["DEFAULT"] = L["Default"]},
                    },
                    chat2 = {
                        name = string.format(L["Chat%dTab"], 2),
                        desc = string.format(L["Toggles chat tab options for chat window %d."], 2),
                        type = "text",
                        get = function() return self.db.profile.cf[2] end,
                        set = function(v)
                            self.db.profile.cf[2] = v
                            self:UpdateAllTabs()
                        end,
                        validate = {["ALWAYS"] = L["Always Visible"], ["HIDDEN"] = L["Hidden"], ["DEFAULT"] = L["Default"]},
                    },
                    chat3 = {
                        name = string.format(L["Chat%dTab"], 3),
                        desc = string.format(L["Toggles chat tab options for chat window %d."], 3),
                        type = "text",
                        get = function() return self.db.profile.cf[3] end,
                        set = function(v)
                            self.db.profile.cf[3] = v
                            self:UpdateAllTabs()
                        end,
                        validate = {["ALWAYS"] = L["Always Visible"], ["HIDDEN"] = L["Hidden"], ["DEFAULT"] = L["Default"]},
                    },
                    chat4 = {
                        name = string.format(L["Chat%dTab"], 4),
                        desc = string.format(L["Toggles chat tab options for chat window %d."], 4),
                        type = "text",
                        get = function() return self.db.profile.cf[4] end,
                        set = function(v)
                            self.db.profile.cf[4] = v
                            self:UpdateAllTabs()
                        end,
                        validate = {["ALWAYS"] = L["Always Visible"], ["HIDDEN"] = L["Hidden"], ["DEFAULT"] = L["Default"]},
                    },
                    chat5 = {
                        name = string.format(L["Chat%dTab"], 5),
                        desc = string.format(L["Toggles chat tab options for chat window %d."], 5),
                        type = "text",
                        get = function() return self.db.profile.cf[5] end,
                        set = function(v)
                            self.db.profile.cf[5] = v
                            self:UpdateAllTabs()
                        end,
                        validate = {["ALWAYS"] = L["Always Visible"], ["HIDDEN"] = L["Hidden"], ["DEFAULT"] = L["Default"]},
                    },
                    chat6 = {
                        name = string.format(L["Chat%dTab"], 6),
                        desc = string.format(L["Toggles chat tab options for chat window %d."], 6),
                        type = "text",
                        get = function() return self.db.profile.cf[6] end,
                        set = function(v)
                            self.db.profile.cf[6] = v
                            self:UpdateAllTabs()
                        end,
                        validate = {["ALWAYS"] = L["Always Visible"], ["HIDDEN"] = L["Hidden"], ["DEFAULT"] = L["Default"]},
                    },
                    chat7 = {
                        name = string.format(L["Chat%dTab"], 7),
                        desc = string.format(L["Toggles chat tab options for chat window %d."], 7),
                        type = "text",
                        get = function() return self.db.profile.cf[7] end,
                        set = function(v)
                           self.db.profile.cf[7] = v
                           self:UpdateAllTabs()
                        end,
                        validate = {["ALWAYS"] = L["Always Visible"], ["HIDDEN"] = L["Hidden"], ["DEFAULT"] = L["Default"]},
                    },
                },
            },
            activealpha = {
                name = L["Active Alpha"],
                desc = L["Sets alpha of chat tab for active chat frame."],
                type = "range",
                get = function() return self.db.profile.activealpha end,
                set = function(v) self.db.profile.activealpha = v self:UpdateAllTabs() end,
                min = 0.1,
                max = 1,
                step = 0.1,
                order = 200,
            },
            notactivealpha = {
                name = L["Not Active Alpha"],
                desc = L["Sets alpha of chat tab for not active chat frame."],
                type = "range",
                get = function() return self.db.profile.notactivealpha end,
                set = function(v) self.db.profile.notactivealpha = v self:UpdateAllTabs() end,
                min = 0.1,
                max = 1,
                step = 0.1,
                order = 210,
            },
        }
    }
    return self.moduleOptions
end

--[[------------------------------------------------
    Module Event Functions
------------------------------------------------]]--

-- things to do when the module is enabled
function Prat_ChatTabs:OnModuleEnable()
    self:SecureHook("FCF_FlashTab")
    self:InstallHooks()
    self:UpdateAllTabs()
end

-- things to do when the module is enabled
function Prat_ChatTabs:OnModuleDisable()
    self:RemoveHooks()
end






--[[------------------------------------------------
    Core Functions
------------------------------------------------]]--


function Prat_ChatTabs:InstallHooks()
    for _, v in ipairs(Prat.Frames) do
        local cftab = getglobal(v:GetName().."Tab")
        cftab:SetScript("OnShow", function(...) Prat_ChatTabs:OnTabShow(cftab, ...) end)
        cftab:SetScript("OnHide", function(...) Prat_ChatTabs:OnTabHide(cftab, ...) end)
    end
end
function Prat_ChatTabs:RemoveHooks()
    for _, v in ipairs(Prat.Frames) do
        local cftab = getglobal(v:GetName().."Tab")
        cftab:SetScript("OnShow", function(...) end)
        cftab:SetScript("OnHide", function(...) end)
    end
    
    -- unhook functions
    self:UnhookAll()    
end


function Prat_ChatTabs:UpdateAllTabs()
    for i = 1,FCF_GetNumActiveChatFrames() do
        local chatTab = getglobal("ChatFrame"..i.."Tab")
        chatTab:Show()
        chatTab:Hide()
        FloatingChatFrame_Update(i) 
    end
end

function Prat_ChatTabs:OnTabShow(tab, ...)
    self:Debug("OnTabShow", tab:GetName())

    local p = self.db.profile    
    local i = tab:GetID()   
    
    if (p.mode == "ALL" and p.cfall == "HIDDEN") or
       (p.mode == "INDIVIDUAL" and p.cf[i] == "HIDDEN") then 
       tab:Hide()
       self:Debug("Hiding "..tab:GetName())
    end  
end

function Prat_ChatTabs:OnTabHide(tab, ...)
    self:Debug("OnTabHide", tab:GetName())
    
    local p = self.db.profile
    local i = tab:GetID()   
    
    if (p.mode == "ALL" and p.cfall == "ALWAYS") or
       (p.mode == "INDIVIDUAL" and p.cf[i] == "ALWAYS") then 
        tab:Show()
        if SELECTED_CHAT_FRAME:GetID() == i then
            tab:SetAlpha(p.activealpha)
        else
            tab:SetAlpha(p.notactivealpha)
        end
       self:Debug("Showing "..tab:GetName())
    end      
end


function Prat_ChatTabs:OnModuleOnUpdateHook(this, elapsed)
-- removed bad code
end

function Prat_ChatTabs:FCF_FlashTab()
    local i = this:GetID()
    local p = self.db.profile
    
    if p.disableflash or 
        (p.mode == "ALL" and p.cfall == "HIDDEN") or
        (p.mode == "INDIVIDUAL" and p.cf[i] == "HIDDEN") then
        
        UIFrameFlashStop(getglobal(this:GetName().."TabFlash"))
    end
end